import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import validate as val

class DayaheadClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        payload = {}
        headers = {
        'Cookie': 'TS01f69930=01cbc7c0b229af3f9e170f80092f828abac28c9cacff2f44fbd6391713e0e3f0af97eecc2694f5fc77aefc033595cc62fe9c469b52'
        }

        response = requests.request("GET", url, headers=headers, data = payload)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data

    def mcp_interim(self, date):

        '''
        This function returns 3 lists including;
            -Date list for specified date as first item.
            -Hour list as second item.
            -Interim MCP values for specified date as third item.

        Parameters:

        Date: Specific date in YYYY-MM-DD format.

        '''
        val.date_format_check(date)

        query = "market/day-ahead-interim-mcp?date="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        price_list = []
        hour_list = []

        for index, item in enumerate(response_list):
            date_list.append(date)
            hour_list.append(index)
            price_list.append(item['marketTradePrice'])

        return date_list, hour_list, price_list

    def mcp(self, startDate, endDate):

        '''
        This function returns 2 lists including;
            -Datetime list that covers the range of startDate and endDate parameters as first item.
            -MCP values for specified range of dates as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "market/day-ahead-mcp?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        datetime_list = []
        price_list = []

        for item_count in response_list:
            price_list.append(item_count['price'])
            datetime_list.append(item_count['date'])

        return datetime_list, price_list

    def diff_fund(self, startDate, endDate, is_statistic):

        '''
        This function returns two different lists according to is_statistic parameter, which includes;
            -Statistical results for the range of specified dates.
            -Date, originatingFromBids, originatingFromOffers, originatingFromRounding and
            total values for the range of specified dates.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        is_statistic: An integer or string value shoul be 1 or TRUE.

        '''

        val.date_check(startDate, endDate)

        query = "market/day-ahead-diff-funds?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]
        second_key_name = key_list[1]

        if is_statistic == 1 or is_statistic == 'TRUE' :
            response_list = json_result['body'][f'{second_key_name}']

        else:
            response_list = json_result['body'][f'{key_name}']

        return response_list

    def block_amount(self, startDate, endDate, is_statistic):
        '''
        This function returns 4 different lists according to is_statistic parameter, which includes;
            -Statistical results for the range of specified dates.
            -Datetime values for the range of specified dates as firts item.
            -Block Sell values for the range of specified dates as second item.
            -Block Buy values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        is_statistic: An integer or string value shoul be 1 or TRUE.

        Warning! For the "MATCHED" block amounts look for block_amount_matched

        '''

        val.date_check(startDate, endDate)

        query = "market/amount-of-block?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]
        second_key_name = key_list[1]

        if is_statistic == 1 or is_statistic == 'TRUE' :
            response_list = json_result['body'][f'{second_key_name}']

            return response_list

        else:
            response_list = json_result['body'][f'{key_name}']

            date_list = []
            purch_amount_list = []
            sale_amount_list = []

            for item in response_list:
                date_list.append(item['date'])
                purch_amount_list.append(item['amountOfPurchasingTowardsBlock'])
                sale_amount_list.append(item['amountOfSalesTowardsBlock'])

            return date_list, sale_amount_list, purch_amount_list


    def block_amount_matched(self, startDate, endDate, is_statistic):
        '''
        This function returns 4 different lists according to is_statistic parameter, which includes;
            -Statistical results for the range of specified dates.
            -Datetime values for the range of specified dates as firts item.
            -Matched Block Sell values for the range of specified dates as second item.
            -Matched Block Buy values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        is_statistic: An integer or string value shoul be 1 or TRUE.

        Warning! For the "NOT MATCHED" block amounts look for block_amount.

        '''

        val.date_check(startDate, endDate)

        query = "market/amount-of-block?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]
        second_key_name = key_list[1]

        if is_statistic == 1 or is_statistic == 'TRUE' :
            response_list = json_result['body'][f'{second_key_name}']

            return response_list

        else:
            response_list = json_result['body'][f'{key_name}']

            date_list = []
            macthed_purch_amount_list = []
            matched_sale_amount_list = []

            for item in response_list:
                date_list.append(item['date'])
                macthed_purch_amount_list.append(item['amountOfPurchasingTowardsMatchBlock'])
                matched_sale_amount_list.append(item['amountOfSalesTowardsMatchBlock'])

            return date_list, matched_sale_amount_list, macthed_purch_amount_list


    def supply_demand_curve(self, date):

        '''
        This function returns 4 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Price values for the range of specified dates as second item.
            -Supply amount values for the range of specified dates as third item.
            -Demand amount values for the range of specified dates as third item.

        Parameters:

        date: Specific date in YYYY-MM-DD format.

        '''

        val.date_format_check(date)

        query = "market/supply-demand-curve?period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        price_list = []
        supply_list = []
        demand_list = []

        for item in response_list:
            date_list.append(item['date'])
            price_list.append(item['price'])
            supply_list.append(item['supply'])
            demand_list.append(item['demand'])

        return date_list, price_list, supply_list, demand_list


    def bilateralContract(self, startDate, endDate):

        '''
        This function returns 3 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Quantity values for the range of specified dates as second item.
            -Next hour values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "market/bilateral-contract?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        quantity_list = []
        next_hour_datelist = []

        for item in response_list:
            date_list.append(item['date'])
            quantity_list.append(item['quantity'])
            next_hour_datelist.append(item['nextHour'])

        return date_list, quantity_list, next_hour_datelist

    def bilateralContract_all(self, startDate, endDate, eic=None):

        '''
        This function returns 3 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Bid Quantity values for the range of specified dates as second item.
            -Ask Quantity values for the range of specified dates as third item.

        Parameters:

        eic (Optional): A code for the specific company e.g: "40X000000009447G".
        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        if eic != None:
            query = "market/bilateral-contract?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&eic="+f'{eic}'
        else:
            query = "market/bilateral-contract?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        quantity_bid = []
        quantity_ask = []

        for item in response_list:
            date_list.append(item['date'])
            quantity_bid.append(item['quantityBid'])
            quantity_ask.append(item['quantityBidAsk'])

        return date_list, quantity_bid, quantity_ask


    def market_income_summary(self, period, startDate, endDate):

        '''
        This function returns 3 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Income values for the range of specified dates as second item.
            -Period information for the range of specified dates as third item.
            -Period type information for the range of specified dates as fourth item.

        Parameters:

        period: Period type shoul be defined as DAILY, WEEKLY, MONTHLY, or PERIODIC.
        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "market/day-ahead-market-income-summary?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&period="+f'{period}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        income_list = []
        period_list = []
        period_type_list = []

        for item in response_list:
            date_list.append(item['date'])
            income_list.append(item['income'])
            period_list.append(item['period'])
            period_type_list.append(item['periodTpye'])

        return date_list, income_list, period_list, period_type_list


    def dayahead_trade_volume(self, startDate, endDate):

        '''
        This function returns 3 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Bid trade volume values for the range of specified dates as second item.
            -Ask trade volume values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Warning! Bid and ask trade volumes are MATCHED dayahead trade volumes. For ask and bid OFFERS look for dayahead_market_volume.

        '''

        val.date_check(startDate, endDate)

        query = "market/day-ahead-market-trade-volume?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        bid_volume_list = []
        ask_volume_list = []

        for item in response_list:
            date_list.append(item['date'])
            bid_volume_list.append(item['volumeOfBid'])
            ask_volume_list.append(item['volumeOfAsk'])

        return date_list, bid_volume_list, ask_volume_list
    
    def dayahead_market_volume(self, startDate, endDate, eic=None):

        '''
        This function returns 3 different which includes;
            -Returns a dictionary which includes following values;
                -date
                -quantityOfAsk
                -volume
                -quantityOfBid
                -priceIndependentBid
                -priceIndependentOffer
                -blockBid
                -blockOffer
                -matchedBids
                -matchedOffers

        Parameters:

        eic (Optional): A code for the specific company e.g: "40X000000009447G".
        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Warning! Bid and ask trade volumes are MATCHED dayahead trade volumes. For ask and bid OFFERS look for dayahead_market_volume.

        '''

        val.date_check(startDate, endDate)

        if eic != None:

            query = "market/day-ahead-market-volume?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&eic="+f'{eic}'
        else:
            query = "market/day-ahead-market-volume?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        response_list = json_result['body']['dayAheadMarketVolumeList']
        
        return response_list

    def imbalance_hourly(self, startDate, endDate):

        '''
        This function returns 3 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Hourly total positive imbalance values for the range of specified dates as second item.
            -Hourly total negative imbalance values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "market/energy-imbalance-hourly?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        postive_imb_list = []
        negative_imb_list = []

        for item in response_list:
            date_list.append(item['date'])
            postive_imb_list.append(item['positiveImbalance'])
            negative_imb_list.append(item['negativeImbalance'])

        return date_list, postive_imb_list, negative_imb_list

    def imbalance_monthly(self, startDate, endDate):

        '''
        This function returns 3 different which includes;
            -Datetime values for the range of specified dates as firts item.
            -Montly total positive imbalance values for the range of specified dates as second item.
            -Montly total negative imbalance values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "market/energy-imbalance-hourly?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        postive_imb_list = []
        negative_imb_list = []

        for item in response_list:
            date_list.append(item['date'])
            postive_imb_list.append(item['positiveImbalance'])
            negative_imb_list.append(item['negativeImbalance'])

        return date_list, postive_imb_list, negative_imb_list
    
    def imbalance_amount(self, startDate, endDate):

        '''
        This function returns 3 different which includes;
            -Date values for the range of specified dates as firts item.
            -Time values for the range of specified dates as firts item.
            -Total positive imbalance amount values for the range of specified dates as second item.
            -Total negative imbalance amount values for the range of specified dates as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "market/imbalance-amount?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        time_list = []
        postive_imb_list = []
        negative_imb_list = []

        for item in response_list:
            date_list.append(item['date'])
            time_list.append(item['time'])
            postive_imb_list.append(item['positiveImbalance'])
            negative_imb_list.append(item['negativeImbalance'])

        return date_list, time_list, postive_imb_list, negative_imb_list

dayahead = DayaheadClient()



from datetime import datetime

def date_check(startDate, endDate):

    date_format_check(startDate)
    date_format_check(endDate)

    startDate = datetime.strptime(startDate, '%Y-%m-%d')
    endDate = datetime.strptime(endDate, '%Y-%m-%d')

    if startDate > endDate:
        raise Exception('starDate parameter has to be less than endDate parameter!')
    else:
        pass

def date_format_check(date):

    try:
        datetime.strptime(date, '%Y-%m-%d')
    except ValueError:
        raise ValueError('Date format should be YYYY-MM-DD ')

def uevcb_eic_check(code):
    
    if code != None:

        key_w = code[2]
        
        if key_w == 'W' and len(code) == 16:
            pass
        else:
            raise Exception('EIC Code should be include W and 16 characters long E.g: 40W000000026808R')
    else:
        pass

def org_eic_check(code):
    
    if code != None:
        
        key_x = code[2]
        
        if key_x == 'X' and len(code) == 16:
            pass
        else:
            raise Exception('EIC Code should be include X and 16 characters long E.g: 40X000000003585Y')
    else:
        pass





import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import dayaheadClient as dc
from transparency_epias.markets import validate as val
from transparency_epias.markets import balancingPowerClient as bpc
from transparency_epias.markets import intradayClient as ic
from transparency_epias.consumption import consumptionClient
import xlsxwriter 
from transparency_epias.markets import validate as val

class reportsClient:

    def daily_excel_export(self):
        
        current_date = datetime.today()

        current_date_formatted = datetime.strftime(current_date,'%Y-%m-%d')

        int_announce_hour = 13
        announce_hour = 14

        if current_date.hour >= int_announce_hour and current_date.hour < announce_hour:
            current_date = current_date + timedelta(days=1)
            current_date_formatted = datetime.strftime(current_date,'%Y-%m-%d')
            mcp_list = dc.dayahead.mcp_interim(current_date_formatted)
            print('Official MCP values are not published yet, for approved values try again after 2:00 P.M')
            mcp_df = pd.DataFrame({'Date':mcp_list[0],
                                'MCP':mcp_list[2]
                                })
        elif current_date.hour >= 14:
            current_date = current_date + timedelta(days=1)
            current_date_formatted = datetime.strftime(current_date,'%Y-%m-%d')
            mcp_list = dc.dayahead.mcp(current_date_formatted,current_date_formatted)
            mcp_df = pd.DataFrame({'Date':mcp_list[0],
                                'MCP':mcp_list[1]
                                })
        else:
            mcp_list = dc.dayahead.mcp(current_date_formatted,current_date_formatted)
            mcp_df = pd.DataFrame({'Date':mcp_list[0],
                                    'MCP':mcp_list[1]
                                    })

        mcp_df.to_excel(f'daily_mcp_{current_date_formatted}.xlsx')
        print(f'Report Created Successfully! Referance Date {current_date_formatted}')

    def mcp_excel_export(self, startDate, endDate):

        mcp_list = dc.dayahead.mcp(startDate, endDate)

        mcp_df = pd.DataFrame({'Date':mcp_list[0],
                                'MCP':mcp_list[1]
                                })
        mcp_df.to_excel(f'MCP_{startDate}_{endDate}.xlsx')

        print('Report Created Successfully!')

    def compare_price(self):

        current_date = datetime.today()

        yest_date = current_date - timedelta(days=1)

        yest_date_formatted = datetime.strftime(yest_date,'%Y-%m-%d')

        mcp = dc.dayahead.mcp(yest_date_formatted, yest_date_formatted)[1]
        smp = bpc.balancingPower.smp(yest_date_formatted, yest_date_formatted)[2]
        wap = ic.intraday.weighted_average_price(yest_date_formatted, yest_date_formatted)[1]
        date = dc.dayahead.mcp(yest_date_formatted, yest_date_formatted)[0]

        df = pd.DataFrame({'Date':date,
                                'MCP':mcp,
                                'SMP':smp,
                                'WAP':wap,
                                })
        
        # Create a Pandas Excel writer using XlsxWriter as the engine.
        excel_file = 'PriceComparison.xlsx'
        sheet_name = 'Sheet1'

        writer = pd.ExcelWriter(excel_file, engine='xlsxwriter')
        df.to_excel(writer, sheet_name=sheet_name)

        # Access the XlsxWriter workbook and worksheet objects from the dataframe.
        workbook = writer.book
        worksheet = writer.sheets[sheet_name]

        # Create a chart object.
        chart = workbook.add_chart({'type': 'line'})

        # Configure the series of the chart from the dataframe data.

        name_list = ['mcp', 'smp','wap']
        line_list = ['solid','dash_dot','round_dot']

        for i in range(len(name_list)):
            
            col = i+2

            chart.add_series({
                'name': ['Sheet1',0,col],
                'categories': ['Sheet1', 1, 0, 24, 0],
                'values':     ['Sheet1', 1, col, 24, col],
                'line':   {'dash_type': f'{line_list[i]}'}
            })

        # Configure the chart axes.
        chart.set_x_axis({'name': 'Index', 'position_axis': 'on_tick'})
        chart.set_y_axis({'name': 'Value', 'major_gridlines': {'visible': False}})

        # Turn off chart legend. It is on by default in Excel.
        chart.set_legend({'position': 'bottom'})

        # Insert the chart into the worksheet.
        worksheet.insert_chart('G2', chart, {'x_scale': 3, 'y_scale': 1.5})

        # Close the Pandas Excel writer and output the Excel file.
        writer.save()

        print('Price Comparison excel is created!')


reports = reportsClient()


class calendarClient:

    def get_holidays(self, startDate, endDate):

        print('Warning! Holidays date range is between 2016 and 2020. Date range will be updated in future')

        holidaysTR = {'Date':["2016-01-01","2016-04-23","2016-05-01","2016-05-19","2016-07-04","2016-07-05","2016-07-06","2016-07-07","2016-08-30","2016-09-11","2016-09-12","2016-09-13","2016-09-14","2016-09-15","2016-10-28","2016-10-29","2017-01-01","2017-04-23","2017-05-01","2017-05-19","2017-06-24","2017-06-25","2017-06-26","2017-06-27","2017-07-15","2017-08-30","2017-08-31","2017-09-01","2017-09-02","2017-09-03","2017-09-04","2017-10-28","2017-10-29","2018-01-01","2018-04-23","2018-05-01","2018-05-19","2018-06-14","2018-06-15","2018-06-16","2018-06-17","2018-07-15","2018-08-20","2018-08-21","2018-08-22","2018-08-23","2018-08-24","2018-08-30","2018-10-28","2018-10-29","2019-01-01","2019-04-23","2019-05-01","2019-05-19","2019-06-03","2019-06-04","2019-06-05","2019-06-06","2019-07-15","2019-08-10","2019-08-11","2019-08-12","2019-08-13","2019-08-14","2019-08-30","2019-10-28","2019-10-29","2020-01-01","2020-04-23","2020-05-01","2020-05-19","2020-05-23","2020-05-24","2020-05-25","2020-05-26","2020-07-15","2020-07-30","2020-07-31","2020-08-01","2020-08-02","2020-08-03","2020-08-30","2020-10-28","2020-10-29"],
              'Days':[1,23,1,19,4,5,6,7,30,11,12,13,14,15,28,29,1,23,1,19,24,25,26,27,15,30,31,1,2,3,4,28,29,1,23,1,19,14,15,16,17,15,20,21,22,23,24,30,28,29,1,23,1,19,3,4,5,6,15,10,11,12,13,14,30,28,29,1,23,1,19,23,24,25,26,15,30,31,1,2,3,30,28,29],
              'Holidays':["Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Religious Holiday","Eve","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Religious Holiday","Eve","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Eve","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Religious Holiday","Religious Holiday","Religious Holiday","Religious Holiday","Bank Holiday","Eve","Bank Holiday"] }

        val.date_check(startDate,endDate)

        # date list range check for parameters should be added
        date_list = []
        holiday_list = []
        for d_index, dates in enumerate(holidaysTR['Date']):
            if startDate <= dates <= endDate:
                date_list.append(dates)
                holiday_list.append(holidaysTR['Holidays'][d_index])

        return date_list, holiday_list

calendar = calendarClient()





import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import validate as val

class consumptionClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        response = requests.request("GET", url)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data
    
    def consumption_all(self, date):

        '''
        This function returns 4 lists including;
            -Date list for specified date as first item.
            -Total consumption amount as second item.
            -Eligible Customer consumption amount for specified date as third item.
            -Under Supply Liability consumption amount for specified date as third item.

        Parameters:

        Date: Specific date in YYYY-MM-DD format.

        '''
        val.date_format_check(date)

        query = "consumption/consumption?period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        consumption_list = []
        consumption_elig_list = []
        consumption_underSupply_list = []

        for item in response_list:
            date_list.append(item['period'])
            consumption_list.append(item['consumption'])
            consumption_elig_list.append(item['eligibleCustomerConsumption'])
            consumption_underSupply_list.append(item['underSupplyLiabilityConsumption'])

        return date_list, consumption_list, consumption_elig_list, consumption_underSupply_list
    
    def distribution_network(self):

        '''
        This function returns a list including;
            -Distribution 

        Parameters:

        None

        '''

        query = "consumption/distribution"

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        network_list = []

        for item in response_list:
            network_list.append(item['distributionList'])

        return network_list

    # response error -1 
    def distribution_profile(self, date, id=None, meterType=None, profile_group=None):

        '''
        This function returns a dictionary including;
            -Date.
            -Multiplier.

        Parameters:

        Date: Specific date in YYYY-MM-DD format.

        '''
        val.date_format_check(date)

        query = "consumption/distribution-profile?period="+f'{date}'+"&distributionId"+f'{id}'+"&subscriberProfileGroup"+f'{profile_group}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def distribution_company(self):

        '''
        This function returns a list including;
            -Distribution companies' information. 

        Parameters:

        None.

        '''

        query = "consumption/get-distribution-company"

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        company_list = []

        for item in response_list:
            company_list.append(item['distributionCompanyList'])

        return company_list
    
    def outage_planned(self, date, companyId=None, provinceId=None):

        '''
        This function returns a list including;
            -Planned outage amount information. 

        Parameters:

        date: Specific date in YYYY-MM-DD format.
        companyId (Optional) = Specific company id.
        provinceId (Optional) = Specific province id.

        '''

        val.date_format_check(date)

        query = "consumption/get-planned-outage?"+"period="+f'{date}'+"&distributionCompanyId="+f'{companyId}'+"&provinceId="+f'{provinceId}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        outage_list = []

        for item in response_list:
            outage_list.append(item['powerOutageList'])

        return outage_list
    
    def outage_unplanned(self, date, companyId=None, provinceId=None):

        '''
        This function returns a list including;
            -Unplanned outage amount information. 

        Parameters:

        date: Specific date in YYYY-MM-DD format.
        companyId (Optional) = Specific company id.
        provinceId (Optional) = Specific province id.

        '''

        val.date_format_check(date)

        query = "consumption/get-unplanned-outage?"+"period="+f'{date}'+"&distributionCompanyId="+f'{companyId}'+"&provinceId="+f'{provinceId}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        outage_list = []

        for item in response_list:
            outage_list.append(item['powerOutageList'])

        return outage_list
    
    #response error 
    def consumption_forecast(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -date
            -lepAvg
            -lepMax
            -lepMin
            -lepSum

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Warning: Estimation plan is not published by EPIAS.

        '''

        val.date_check(startDate, endDate)

        query = "consumption/load-estimation-plan?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list

    def consumption_realtime(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -Datetime list that covers the range of startDate and endDate parameters as first item. 
            -Actual consumption amount as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "consumption/real-time-consumption?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        consumption_list = []

        for item in response_list:
            date_list.append(item['date'])
            consumption_list.append(item['consumption'])

        return date_list, consumption_list
    
    def consumption_official(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -Datetime list that covers the range of startDate and endDate parameters as first item. 
            -Officially aproved consumption amount as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "consumption/swv?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        consumption_list = []

        for item in response_list:
            date_list.append(item['date'])
            consumption_list.append(item['swv'])

        return date_list, consumption_list
    
    def consumption_supplyLia(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -Datetime list that covers the range of startDate and endDate parameters as first item. 
            -Under supply liablitiy consumption amount as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "consumption/under-supply-liability-consumption?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        swv_list = []

        for item in response_list:
            date_list.append(item['date'])
            swv_list.append(item['swv'])

        return date_list, swv_list
    
    def consumption_eligible(self, date):

        '''
        This function returns a list including;
            -Date information as first item.
            -Eligible consumer consumption amount as second item.

        Parameters:

        date: Specific date in YYYY-MM-DD format.
        companyId (Optional) = Specific company id.
        provinceId (Optional) = Specific province id.

        '''

        val.date_format_check(date)

        query = "consumption/swv-v2?"+"period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        svw2_list = []

        for item in response_list:
            date_list.append(item['vc_gec_trh'])
            svw2_list.append(item['st'])

        return date_list, svw2_list
        
consumption = consumptionClient()


import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import validate as val

class ancillaryServicesClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        payload = {}
        headers = {
        'Cookie': 'TS01f69930=01cbc7c0b229af3f9e170f80092f828abac28c9cacff2f44fbd6391713e0e3f0af97eecc2694f5fc77aefc033595cc62fe9c469b52'
        }

        response = requests.request("GET", url, headers=headers, data = payload)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data

    def pfc_amount(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Date list for specified date as first item.
            -Hour information as second item.
            -Primary Frequancy Reserve Amounts as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "market/pfc-amount?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        hour_list = []
        amount_list = []

        for item in response_list:
            date_list.append(item['effectiveDate'])
            hour_list.append(item['hour'])
            amount_list.append(item['totalAmount'])

        return date_list, hour_list, amount_list

    def pfc_price(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Date list for specified date as first item.
            -Hour information as second item.
            -Primary Frequancy Price values as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "market/pfc-price?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        hour_list = []
        price_list = []

        for item in response_list:
            date_list.append(item['effectiveDate'])
            hour_list.append(item['hour'])
            price_list.append(item['price'])

        return date_list, hour_list, price_list

    def sfc_amount(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Date list for specified date as first item.
            -Hour information as second item.
            -Secondary Frequancy Reserve amounts as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "market/sfc-amount?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        hour_list = []
        amount_list = []

        for item in response_list:
            date_list.append(item['effectiveDate'])
            hour_list.append(item['hour'])
            amount_list.append(item['totalAmount'])

        return date_list, hour_list, amount_list

    def sfc_price(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Date list for specified date as first item.
            -Hour information as second item.
            -Secondary Frequancy Reserve amounts as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "market/sfc-price?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        hour_list = []
        price_list = []

        for item in response_list:
            date_list.append(item['effectiveDate'])
            hour_list.append(item['hour'])
            price_list.append(item['price'])

        return date_list, hour_list, price_list

ancillary = ancillaryServicesClient()


import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import validate as val

class IntradayClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        payload = {}
        headers = {
        'Cookie': 'TS01f69930=01cbc7c0b229af3f9e170f80092f828abac28c9cacff2f44fbd6391713e0e3f0af97eecc2694f5fc77aefc033595cc62fe9c469b52'
        }

        response = requests.request("GET", url, headers=headers, data = payload)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data

    def weighted_average_price(self, startDate, endDate):

        '''
        This function returns 2 lists including;
            -Date list for specified date as first item.
            -Intraday weighted average price values as second item 

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "market/intra-day-aof?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        price_list = []

        for item in response_list:
            date_list.append(item['date'])
            price_list.append(item['price'])

        return date_list, price_list
    
    def income_intraday(self, startDate, endDate):

        '''
        This function returns 2 lists including;
            -Date list for specified range of date as first item.
            -Intraday income values for specified range of date as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "market/intra-day-aof?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        income_list = []

        for item in response_list:
            date_list.append(item['date'])
            income_list.append(item['income'])

        return date_list, income_list

    def block_offer_prices(self, startDate, endDate):

        '''
        This function returns a dictionary which includes the following information;
            -Date
            -Minimum ask price
            -Maximum ask pirce
            -Minimum bid price
            -Maximum bid pirce
            -Minimum matched price
            -Maximum matched price


        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Warning! For hourly offer values look for hourly_offer_prices

        '''
        val.date_check(startDate, endDate)

        query = "market/intra-day-min-max-price?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&offerType=BLOCK"

        json_result = self.get_request_result(query)

        response_list = json_result['body']

        return response_list

    def hourly_offer_prices(self, startDate, endDate):

        '''
        This function returns a dictionary which includes the following information;
            -Date
            -Minimum ask price
            -Maximum ask pirce
            -Minimum bid price
            -Maximum bid pirce
            -Minimum matched price
            -Maximum matched price


        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Warning! For block offer values look for block_offer_prices
    
        '''
        val.date_check(startDate, endDate)

        query = "market/intra-day-min-max-price?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&offerType=HOURLY"

        json_result = self.get_request_result(query)

        response_list = json_result['body']

        return response_list

    def hourly_quantities(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Effective date information as first item.
            -Hourly sell quantities as second item. 
            -Hourly buy quantities as third item.


        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Warning! For block offer values look for block_offer_prices
    
        '''
        val.date_check(startDate, endDate)

        query = "market/intra-day-min-max-price?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        response_list = json_result['body']

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        sell_quantity = []
        buy_quantity = []

        for item in response_list:
            date_list.append(item['effectiveDate'])
            sell_quantity.append(item['blockPurchaseQuantity'])
            buy_quantity.append(item['blockSaleQuantity'])

        return date_list, sell_quantity, buy_quantity

    def trade_history(self, startDate, endDate, contract_type):

        '''
        This function returns 5 lists according to contract_type argument which including:
            -Datetime information as first item.
            -Contract id infromation as second item. 
            -Contract name information as third item.
            -Quantity values as fourth item.
            -Price values as fifth item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        contract_type: Block or Hourly

        '''
        val.date_check(startDate, endDate)

        query = "market/intra-day-trade-history?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        response_list = json_result['body']

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']
        
        hourly_date_list = []
        houry_id_list = []
        hourly_quantity = []
        hourly_price = []
        houryl_contract_list = []

        block_date_list = []
        block_id_list = []
        block_quantity = []
        block_price = []
        block_contract_list = []

        block_key = 'PB'
        
        for item in response_list:
            contract_name = item['conract']
            
            if block_key in contract_name:
                block_quantity.append(item['quantity'])
                block_price.append(item['price'])
                block_id_list.append(item['id'])
                block_date_list.append(item['date'])
                block_contract_list.append(item['conract'])
            else:
                hourly_quantity.append(item['quantity'])
                hourly_price.append(item['price'])
                houry_id_list.append(item['id'])
                hourly_date_list.append(item['date'])
                houryl_contract_list.append(item['conract'])

        if contract_type == 'Block':

            return block_date_list, block_id_list, block_contract_list, block_quantity, block_price

        elif contract_type == 'Hourly':

            return hourly_date_list, houry_id_list, houryl_contract_list, hourly_quantity, hourly_price

        else:
            
            raise Exception('contract_type argument has to be Block or Hourly!')

intraday = IntradayClient()


import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import validate as val

class gasTransmissionClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        response = requests.request("GET", url)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data
    
    def stock_amoutn(self, startDate, endDate):

        '''
        This function returns 2 lists including;
            -Gas day informatin as first item.
            -Pipe stock amount values as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp-transmission/actualization?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        pipe_stock = []

        for item in response_list:
            day_list.append(item['gasDay'])
            pipe_stock.append(item['pipeStock'])

        return day_list, pipe_stock
    
    def transport(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Gas day informatin as first item.
            -Entry nomination amount values as second item.
            -Exit nomination amount values as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp-transmission/transport?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        entry_list = []
        exit_list = []

        for item in response_list:
            day_list.append(item['gasDay'])
            entry_list.append(item['entryNomination'])
            exit_list.append(item['exitNomination'])

        return day_list, entry_list, exit_list
    
gas_transmission = gasTransmissionClient()


class gasClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        response = requests.request("GET", url)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data

    def notification_additional(self, startDate, endDate):

            '''
            This function returns 4 lists including;
                -Date informatin as first item.
                -Id information as second item.
                -Message information as third item.
                -Subject information as fourth item.

            Parameters:

            startDate: Start date in YYYY-MM-DD format.
            endDate: End date in YYYY-MM-DD format.

            '''

            val.date_check(startDate, endDate)

            query = "stp/additional-notification?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

            json_result = self.get_request_result(query)

            key_list = list(json_result['body'].keys())

            key_name = key_list[0]

            response_list = json_result['body'][f'{key_name}']

            date_list = []
            message_list = []
            subject_list = []
            id_list = []

            for item in response_list:
                date_list.append(item['date'])
                message_list.append(item['messageEn'])
                subject_list.append(item['subjectEn'])
                id_list.append(item['id'])

            return date_list, id_list, message_list, subject_list
        
    def allowance(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -GasDay
            -InputDataPyhsical
            -InputDataVirtual
            -NegativeImbalance
            -NegativeImbalanceTradeValue
            -Opsiyonel
            -OutputDataPyhsical
            -OutputDataVirtual
            -PositiveImbalance
            -PositiveImbalanceTradeValue
            -SystemDirection
            -Type

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/allowance?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list

    def balancing_gas_price(self, startDate, endDate):

            '''
            This function returns a dictionary including;
                -additionalBalancingPurchase
                -additionalBalancingSale
                -balancingGasPurchase
                -balancingGasSale
                -finalAbp
                -finalAbs
                -finalBgp
                -finalBgs
                -gasDay


            Parameters:

            startDate: Start date in YYYY-MM-DD format.
            endDate: End date in YYYY-MM-DD format.

            '''

            val.date_check(startDate, endDate)

            query = "stp/balancing-gas-price?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

            json_result = self.get_request_result(query)

            key_list = list(json_result['body'].keys())

            key_name = key_list[0]

            response_list = json_result['body'][f'{key_name}']

            return response_list
    
    # null response
    def bluecode(self, startDate, endDate):

        '''
        This function returns 4 lists including;
            -Gas day informatin as first item.
            -Contract Name information as second item.
            -Amount values as third item.
            -Weighted average values as fourth item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/bluecode-operation?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        contract_list = []
        amount_list = []
        wAvg_list = []

        for item in response_list:
            day_list.append(item['gasDay'])
            contract_list.append(item['contractName'])
            amount_list.append(item['amount'])
            wAvg_list.append(item['weightedAverage'])

        return day_list, contract_list, amount_list, wAvg_list
    
    def contract_amount(self, startDate, endDate, date=None):

        '''
        This function returns 5 lists including;
            -Gas day informatin as first item.
            -Matched quantitiy amount as second item.
            -Period information as third item.
            -Period Type information  as fourth item.
            -Trade volume amount as fifth item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        date (Optional): Specific date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)
        
        if date != None:
            val.date_format_check(date)
        else:
            pass

        query = "stp/bluecode-operation?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        matched_list = []
        period_list = []
        periodType_list = []
        trade_list = []

        for item in response_list:
            day_list.append(item['gasDay'])
            matched_list.append(item['matchingQuantity'])
            period_list.append(item['period'])
            periodType_list.append(item['periodType'])
            trade_list.append(item['tradeValue'])

        return day_list, matched_list, period_list, periodType_list, trade_list
    
    def price_daily(self, startDate, endDate):

        '''
        This function returns 7 lists including;
            -Gas day informatin as first item.
            -Contract name information as second item.
            -Intraday price values as third item.
            -Day after price values as fourth item.
            -Dayahead price values as fifth item.
            -Weighted Average price values as sixth item.
            -Gas reference price values as seventh item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/daily-price?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        contract_list = []
        intraday_list = []
        dayAfter_list = []
        dayahead_list = []
        wavg_list = []
        gas_ref_price = []

        for item in response_list:
            day_list.append(item['gasDay'])
            contract_list.append(item['contractName'])
            intraday_list.append(item['intraDayPrice'])
            dayAfter_list.append(item['dayAfterPrice'])
            dayahead_list.append(item['dayAheadPrice'])
            wavg_list.append(item['weightedAverage'])
            gas_ref_price.append(item['gasReferencePrice'])

        return day_list, contract_list, intraday_list, dayAfter_list, dayahead_list, wavg_list, gas_ref_price
    
    def fourcode(self, startDate, endDate):

        '''
        This function returns 6 lists including;
            -Gas day informatin as first item.
            -Contract name information as second item.
            -Amount values (x1000 sm3) values as third item.
            -Weigthed average values as fourth item.
            

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/fourcode-operation?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        amount_list = []
        contract_list = []
        wavg_list = []

        for item in response_list:
            day_list.append(item['gasDay'])
            contract_list.append(item['contractName'])
            amount_list.append(item['amount'])
            wavg_list.append(item['weightedAverage'])

        return day_list, contract_list, amount_list, wavg_list
    
    def greencode(self, startDate, endDate):

        '''
        This function returns 6 lists including;
            -Gas day informatin as first item.
            -Contract name information as second item.
            -Amount values (x1000 sm3) values as third item.
            -Weigthed average values as fourth item.
            -Transaction date information as fifth item.
            -Contract name information as sixth item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/greencode-operation?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        amount_list = []
        contract_list = []
        wavg_list = []
        transactiondate_list = []
        c_day_list = []

        for item in response_list:
            day_list.append(item['gasDay'])
            contract_list.append(item['contractName'])
            amount_list.append(item['amount'])
            wavg_list.append(item['weightedAverage'])
            transactiondate_list.append(item['transactionDate'])
            c_day_list.append(item['contractGasDay'])

        return day_list, contract_list, amount_list, wavg_list, transactiondate_list, c_day_list
    
    def price_reference(self, startDate, endDate, date=None):

        '''
        This function returns 4 lists including;
            -Gas day informatin as first item.
            -Period information as second item.
            -Period type as third item.
            -Reference price values as fourth item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        date (Optional): Specific date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        if date != None:
            val.date_format_check(date)
        else:
            pass

        query = "stp/grf?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        day_list = []
        period_list = []
        price_list = []
        p_type = []
        

        for item in response_list:
            day_list.append(item['gasDay'])
            period_list.append(item['period'])
            price_list.append(item['price'])
            p_type.append(item['periodType'])

        return day_list, period_list, p_type, price_list
    
    def imbalance_montly(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -negativeImbalance
            -negativeImbalanceTradeValue
            -period
            -positiveImbalance
            -positiveImbalanceTradeValue
            -type

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/imbalance-monthly?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def quantitiy_matched(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -contractMatchingQuantity
            -contractName
            -dayAfterMatchingQuantity
            -dayAheadMatchingQuantity
            -gasDay
            -gasReferenceMatchingQuantity
            -intraDayMatchingQuantity

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/matching-quantity?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list

    def quantitiy_matched_additional(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Quantity amount as first item.
            -Gas day information as second item.
            -Other quantity amount as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/matching-quantity/additional-quantity?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        quant_list = []
        day_list = []
        otherQuant_list = []

        for item in response_list:
            quant_list.append(item['additionalQuantity'])
            day_list.append(item['gasDay'])
            otherQuant_list.append(item['otherQuantity'])

        return quant_list, day_list, otherQuant_list
    
    def price_mobile(self, startDate, endDate):

        '''
        This function returns 5 lists including;
            -Balancing gas purchase amount as first item.
            -Balancing gas sale amount as second item.
            -Gas reference price values as third item.
            -Gas day information as fourth item.
            -Imbalance amount as sixth item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/mobile/price?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        b_purch_list = []
        b_sale_list = []
        gas_refp_list = []
        day_list = []
        imb_list = []

        for item in response_list:
            b_purch_list.append(item['balancingGasPurchase'])
            day_list.append(item['gasDay'])
            b_sale_list.append(item['balancingGasPurchase'])
            gas_refp_list.append(item['otherQuantity'])
            imb_list.append(item['imbalance'])

        return b_purch_list, b_sale_list, gas_refp_list, day_list, imb_list
    
    def orangecode(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -amount
            -contractGasDay
            -contractName
            -gasDay
            -transactionDate
            -weightedAverage

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/orangecode-operation?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    # Price type validation needed!
    def price_stp(self, startDate, endDate, price_type=None):

        '''
        This function returns 4 lists including;
            -Gas day information as first item.
            -Price values as second item.
            -Price type information as third item.
            -State information as fourht item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/price?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&priceType="+f'{price_type}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        g_day_list = []
        price_list = []
        priceType_list = []
        state_list = []

        for item in response_list:
            g_day_list.append(item['gasDay'])
            price_list.append(item['price'])
            priceType_list.append(item['priceType'])
            state_list.append(item['state'])

        return g_day_list, price_list, priceType_list, state_list
    
    def trade_value(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -contractName
            -contractTradeValue
            -dayAfterTradeValue
            -dayAheadTradeValue
            -gasDay
            -gasReferenceTradeValue
            -intraDayTradeValue

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/trade-value?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def transaction_history(self, startDate, endDate):

        '''
        This function returns a dictionary including;
            -contractName
            -id
            -mathcingDate
            -price
            -quantity

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/transaction-history?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def zero_balance(self, startDate, endDate):

        '''
        This function returns 2 lists including;
            -Gas day date information as first item.
            -Zero balance amount as second item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''

        val.date_check(startDate, endDate)

        query = "stp/zero-balance?"+"startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        zero_b_list = []
        g_day_list = []

        for item in response_list:
            zero_b_list.append(item['zeroBalance'])
            g_day_list.append(item['gasDay'])

        return g_day_list, zero_b_list

gas = gasClient()


import pandas as pd 
import requests
import json
from datetime import timedelta 
from datetime import datetime
from transparency_epias.markets import validate as val

class productionClient:

    def get_request_result(self, query):

        #main_url = "https://seffaflik.epias.com.tr/transparency/service/market/"

        url = "https://seffaflik.epias.com.tr/transparency/service/"+query

        response = requests.request("GET", url)

        json_data = json.loads(response.text.encode('utf8'))

        return json_data
    
    def availability(self, startDate, endDate, orgEic=None, uevcbEic=None):

        '''
        This function returns a dictionary including following information;
            -Date (tarih)
            -Total (toplam)
            -Natural Gas (dogalgaz)
            -Wind (ruzgar)
            -Brown Coal (linyit)
            -Bituminous Coal (tasKomur)
            -Import Coal (ithalKomur)
            -Fuel (fuelOil)
            -Geothermal (jeotermal)
            -Dam (barajli)
            -Naphtha-based (nafta)
            -Biomass (biokutle)
            -River (akarsu)
            -Other (diger)

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        orgEic (Optional): Organization EIC Code information e.g: 40X000000003585Y.
        uevcbEic (Optional): Organization EIC Code information e.g: 40W000000026808R.

        Note: For certain power plant availability information, both orgEic and uevcbEic parameters should pass as an argument.

        '''
        val.date_check(startDate, endDate)

        if orgEic == None and uevcbEic != None:
            raise Exception('Please provide organization eic cide with uevbEic code to get plant availability informaton' )
        else:
            pass

        val.uevcb_eic_check(uevcbEic)
        val.org_eic_check(orgEic)

        query = "production/aic?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&organizationEIC="+f'{orgEic}'+"&uevcbEIC="+f'{uevcbEic}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list

    def daily_production_plan(self, startDate, endDate, orgEic=None, uevcbEic=None):

        '''
        This function returns a dictionary including following information;
            -Date (tarih)
            -Hour (saat)
            -Total (toplam)
            -Natural Gas (dogalgaz)
            -Wind (ruzgar)
            -Brown Coal (linyit)
            -Bituminous Coal (tasKomur)
            -Import Coal (ithalKomur)
            -Fuel (fuelOil)
            -Geothermal (jeotermal)
            -Dam (barajli)
            -Naphtha-based (nafta)
            -Biomass (biokutle)
            -River (akarsu)
            -Other (diger)

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        orgEic (Optional): Organization EIC Code information e.g: 40X000000003585Y.
        uevcbEic (Optional): Organization EIC Code information e.g: 40W000000026808R.

        Note: For certain power plant availability information, both orgEic and uevcbEic parameters should pass as an argument.

        '''
        val.date_check(startDate, endDate)

        if orgEic == None and uevcbEic != None:
            raise Exception('Please provide organization eic cide with uevbEic code to get plant availability informaton' )
        else:
            pass

        val.uevcb_eic_check(uevcbEic)
        val.org_eic_check(orgEic)

        query = "production/dpp?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&organizationEIC="+f'{orgEic}'+"&uevcbEIC="+f'{uevcbEic}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def daily_production_plan_total(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -Date infromation.
            -Total Daily production plan for specified datetime.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/final-dpp?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def back_charge(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -Back charge credit amount (gddk Credit Amount)
            -Back charge debt amount (gddkDebtAmount)
            -Net back charge amount (gddkNetAmount)

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/gddk-amount?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def installed_cap(self, date):

        '''
        This function returns a dictionary including following information;
            -Period information.
            -Capacity type information.
            -Capacity amount.

        Parameters:

        date: Date in YYYY-MM-DD format.

        Note: For plants in feed-in tarriffs look for install_cap_fit
        '''
        val.date_format_check(date)

        query = "production/installed-capacity?period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}'][1]

        return response_list
    
    def installed_cap_fit(self, date):

        '''
        This function returns a dictionary including following information;
            -Period information.
            -Capacity type information.
            -Capacity amount.

        Parameters:

        date: Date in YYYY-MM-DD format.

        Note: For all plants look for install_cap.
        '''
        val.date_format_check(date)

        query = "production/installed-capacity?period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}'][0]

        return response_list

    def installed_cap_renewables(self, date):

        '''
        This function returns a dictionary including following information;
            -Capacity type id information.
            -Period information.
            -Capacity type information.
            -Capacity amount.

        Parameters:

        date: Date in YYYY-MM-DD format.

        '''
        val.date_format_check(date)

        query = "production/installed-capacity-of-renewable?period="+f'{date}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def real_time_gen(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -Biomass
            -BlackCoal
            -DammedHydro
            -Date
            -Fueloil
            -GasOil
            -Geothermal
            -ImportCoal
            -ImportExport
            -Lignite
            -Lng
            -Naphta
            -NaturalGas
            -Nuclear (Nucklear **There is a typo on the response)
            -River
            -Sun
            -Total
            -Wind

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/real-time-generation?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def real_time_gen_renewabales(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -Biogas
            -Biomass
            -CanalType
            -Date
            -Geothermal
            -Lfg
            -Others
            -Reservoir
            -RiverType
            -Sun
            -Total
            -Wind

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/renewable-sm-licensed-real-time-generation?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def unit_cost_fit(self, startDate, endDate):

        '''
        This function returns 3 lists including;
            -Unit Cost values as first item.
            -Version date information as second item.
            -Period date information as third item.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/renewable-sm-unit-cost?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        unit_cost_list = []
        version_list = []
        period_list = []

        for x in range(len(response_list)):
            unit_cost_list.append(response_list[x]['unitCost'])
            version_list.append(response_list[x]['id']['donem'])
            period_list.append(response_list[x]['id']['versiyon'])
        
        return unit_cost_list, version_list, period_list
    
    def generation_official_unlicenced_fit(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -Date
            -Total
            -Wind
            -Biogas
            -CanalType
            -Biomass
            -Sun
            -Others

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/renewable-unlicenced-generation-amount?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def fit_cost_total(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -LicenseExemptCost
            -Period
            -PortfolioIncome
            -ReneablesCost (RenewableCost **There is a typo in the response)
            -RenewablesTotalCost
            -UnitCost

            
        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/renewables-support?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list

    # response error code -1
    def daily_production_plan_updated(self, startDate, endDate, orgEic=None, uevcbEic=None):

        '''
        This function returns a dictionary including following information;
            -Date (tarih)
            -Hour (saat)
            -Total (toplam)
            -Natural Gas (dogalgaz)
            -Wind (ruzgar)
            -Brown Coal (linyit)
            -Bituminous Coal (tasKomur)
            -Import Coal (ithalKomur)
            -Fuel (fuelOil)
            -Geothermal (jeotermal)
            -Dam (barajli)
            -Naphtha-based (nafta)
            -Biomass (biokutle)
            -River (akarsu)
            -Other (diger)

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.
        orgEic (Optional): Organization EIC Code information e.g: 40X000000003585Y.
        uevcbEic (Optional): Organization EIC Code information e.g: 40W000000026808R.

        Note: For certain power plant availability information, both orgEic and uevcbEic parameters should pass as an argument.

        '''
        val.date_check(startDate, endDate)

        if orgEic == None and uevcbEic != None:
            raise Exception('Please provide organization eic cide with uevbEic code to get plant availability informaton' )
        else:
            pass

        val.uevcb_eic_check(uevcbEic)
        val.org_eic_check(orgEic)

        query = "production/sbfgp?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'+"&organizationEIC="+f'{orgEic}'+"&uevcbEIC="+f'{uevcbEic}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    #null response
    def plant_notifications(self, startDate, endDate):

        '''
        This function returns a dictionary including following information;
            -CapacityAtCaseTime
            -CaseAddDate
            -CaseEndDate
            -CaseStartDate
            -CaseVaguenessTime
            -City
            -CityId
            -FaultDetails
            -FuelType
            -FuelTypeId
            -Id
            -MessageType
            -OperatorPower
            -PowerPlantName
            -Reason
            -Region
            -UevcbName
            -validityStatus

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        '''
        val.date_check(startDate, endDate)

        query = "production/urgent-market-message?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        return response_list
    
    def production_total_official(self, startDate, endDate):

        '''
        This function returns two lists including;
            -Datetime information.
            -Officially aproved total production amount.

        Parameters:

        startDate: Start date in YYYY-MM-DD format.
        endDate: End date in YYYY-MM-DD format.

        Note:
        '''
        val.date_check(startDate, endDate)

        query = "production/ssv?startDate="+f'{startDate}'+"&endDate="+f'{endDate}'

        json_result = self.get_request_result(query)

        key_list = list(json_result['body'].keys())

        key_name = key_list[0]

        response_list = json_result['body'][f'{key_name}']

        date_list = []
        prod_list = []

        for item in response_list:
            date_list.append(item['date'])
            prod_list.append(item['ssv'])

        return date_list, prod_list

production = productionClient()